/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/08/10 10:10:06
 */

import { Component, html, raw } from 'wkit'

import createWebHashHistory from './hash-router.js'
import createWebHistory from './modern-router.js'

export { createWebHashHistory, createWebHistory }

class Router extends Component {
  static props = {
    keepAlive: false,
    transition: false,
    current: {
      type: String,
      default: '',
      attribute: false,
      observer(v, old) {
        if (this.keepAlive && v) {
          if (old && this.$refs[old]) {
            this.$refs[old].deactivated()
          }
          this.$refs[v]?.$animate()
          this.$refs[v]?.activated()
        }
      }
    }
  }

  #views = []

  created() {
    window.__wkitd__.set('ROUTER_VIEW', this)
  }

  sync(views) {
    this.#views = views
  }

  render() {
    let option = {
      immediate: true,
      custom: [
        { transform: 'translateX(-32px)', opacity: 0 },
        { transform: 'translateX(0)', opacity: 1 }
      ]
    }
    if (this.keepAlive) {
      let template = this.#views.map(it => [
        this.transition
          ? `<${it} ref="${it}" :__keep_alive__="%s" #animation="%s" style="%s"></${it}>`
          : `<${it} ref="${it}" :__keep_alive__="%s" style=%s></${it}>`,
        [
          this.current === it,
          { ...option, immediate: this.current === it },
          this.current === it ? '' : 'display:none'
        ]
      ])

      return raw(
        template.map(it => it[0]).join(''),
        template.map(it => it[1]).flat()
      )
    } else {
      if (this.current) {
        if (this.transition) {
          return raw(`<${this.current} #animation="%s"></${this.current}>`, [
            option
          ])
        }
        return raw(`<${this.current}></${this.current}>`)
      }
    }
  }
}

if (!customElements.get('router-view')) {
  customElements.define('router-view', Router)
}

export function createRouter(
  { history = createWebHashHistory, routes = [] } = {},
  options
) {
  let $router = history(options)

  window.__wkitd__.set('$router', $router)
  Component.prototype.$router = $router

  $router.addRoute(routes)

  return function () {
    return $router
  }
}

export function getRouter() {
  return window.__wkitd__.get('$router')
}

export function getCurrentPage() {
  return getRouter().route
}
